data;
infile '/home/timothyhaas/lionb.dat';
length id $12 date $20 age $9 pride $20;
/*
ID_NO DATE Collared=1_No_Collar=0 SEX AGE_CLASS GROUP_ASSOCIATION PRIDE Detection_Method:_1=Collar;_0=Opportunistic
*/
input id $ date $ radcoll gender $ age $ group pride $ dmthd;
run;

data;
set;
date = tranwrd(date,"_","");
date = tranwrd(date,"January","Jan");
date = tranwrd(date,"February","Feb");
date = tranwrd(date,"March","Mar");
date = tranwrd(date,"April","Apr");
date = tranwrd(date,"June","Jun");
date = tranwrd(date,"July","Jul");
date = tranwrd(date,"August","Aug");
date = tranwrd(date,"September","Sep");
date = tranwrd(date,"October","Oct");
date = tranwrd(date,"November","Nov");
date = tranwrd(date,"December","Dec");
date = tranwrd(date,"2013","13");
date = tranwrd(date,"2014","14");
date = tranwrd(date,"2015","15");

cdate = input(date,date9.);

if '01Jan13'd <= cdate <= '01Sep13'd then ocsn = 'ocsn1';
if '02Sep13'd <= cdate <= '01Jan14'd then ocsn = 'ocsn2';
if '02Jan14'd <= cdate <= '01May14'd then ocsn = 'ocsn3';
if '02May14'd <= cdate <= '01Sep14'd then ocsn = 'ocsn4';
if '02Sep14'd <= cdate <= '01Jan15'd then ocsn = 'ocsn5';
if '02Jan15'd <= cdate <= '01Jul15'd then ocsn = 'ocsn6';

/* Recode unknown gender assignments. */

if id = "14FY02-C1" then gender = "FEMALE";
if id = "14FY02-C3" then gender = "MALE";

/* Recode transgender assignments. */

if id = "14FY01-C1" then gender = "MALE";
if id = "13F22-C1" then gender = "FEMALE";
if id = "13F22-C3" then gender = "FEMALE";
if id = "13FX25-C2" then gender = "MALE";

sightnm = 1;
run;

/* Correct transgender lions, and create an integer id. */

proc sort;
by id cdate;
run;

data;
set;
retain idsve iid gendersve;
if _N_ = 1 then do;
   idsve = id;
   iid = 1;
   gendersve = gender;
end;

if id = idsve & gender ^= gendersve then gender = gendersve;

if id ^= idsve & gender ^= 'UNKNOWN' then do;
   iid = iid + 1;
end;
idsve = id;
gendersve = gender;

if gender = 'MALE' then gndr = 1;
if gender = 'FEMALE' then gndr = 0;

if age = 'CUB' then ageclss = 1;
if age = 'JUVENILE' then ageclss = 2;
if age = 'SUB-ADULT' then ageclss = 3;
if age = 'ADULT' then ageclss = 4;

/* Delete lions with unknown gender. */

if gender ^= 'UNKNOWN' then output;

keep id iid radcoll gender gndr age ageclss pride cdate dmthd;
run;

proc sort;
by iid cdate;
run;

data lions;
set;
run;

/*
proc print;
var iid gender ageclss cdate;
run;
quit;
*/

/* ------------------- Main program ----------------------------- */

proc iml;
reset noname printadv=0 fw=8;

/* Read-in data. */

use lions;
read all;

/* Build capture time history array. */

ntimes = nrow(iid);
gender = j(ntimes, 1, 0);
age = j(ntimes, 30, 0);
dmthd = j(ntimes, 30, 0);
obshdat = j(ntimes, 30, 0);
m = j(ntimes, 1, 0);
mindate = min(cdate);
tau = max(cdate) - mindate;

do k = 1 to ntimes;
   id = iid[k];
   obshdat[id,1] = id;
   gender[id] = gndr[k];

   m[id] = m[id] + 1;
   age[id,m[id]] = ageclss[k];
   dmthd[id,m[id]] = dmthd[k];
   obshdat[id,m[id] + 1] = cdate[k] - mindate;
end;

obsn = max(iid);
n = obsn;
maxm = max(m);
hdat = obshdat;

/* Compute empirical cub survival. */

nlive = 0;
ncubs = 0;
do i = 1 to n;
   if gender[i] = 0 & age[i,1] = 1 then do;
      ncubs = ncubs + 1;
      survd = 0;
      do j = 1 to m[i];
         if age[i,j] > 1 then survd = 1;
      end;
      if survd = 1 then nlive = nlive + 1;
   end;
end;
cubsurvp = nlive / ncubs;
print "Empirical female cub survival probability= " cubsurvp;

/* Create a data set of first detection time and draw its histogram.
ftime = j(n, 1, 0);
j = 1;
do i = 1 to n;
   if gender[i] = 1 & age[i,1] > 1 then do;
      ftime[j] = obshdat[i,1];
      j = j + 1;
   end;
end;

create firsttime from ftime [colname="mfirsttime"];
append from ftime;
close firsttime;
quit;
ods trace on;
ods pdf file="mfirst.pdf";
ods select Histogram;
proc univariate data=firsttime noprint;
var mfirsttime;
histogram / name='Histogram'; 
run;
ods pdf close;
quit;
*/

/* Artificial data set.
do i = 1 to n;
   artage = age[i,m[i]];
   artdmthd = dmthd[i,m[i]];
   m[i] = 10;
   do j = 1 to m[i];
      age[i,j] = artage;
      dmthd[i,j] = artdmthd;
      hdat[i,(j + 1)] = (j / 10.) # tau;
   end;
end;
*/

/* Define goodness of fit time interval cells. */

tcell = j(6, 2, 0);
tcell[1,1] = 0.;
tcell[1,2] = '01Sep13'd - mindate;

tcell[2,1] = '02Sep13'd - mindate;
tcell[2,2] = '01Jan14'd - mindate;

tcell[3,1] = '02Jan14'd - mindate;
tcell[3,2] = '01May14'd - mindate;

tcell[4,1] = '02May14'd - mindate;
tcell[4,2] = '01Sep14'd - mindate;

tcell[5,1] = '02Sep14'd - mindate;
tcell[5,2] = '01Jan15'd - mindate;

tcell[6,1] = '02Jan15'd - mindate;
tcell[6,2] = tau;
tau = tau + 3. # (tcell[6,2] - tcell[6,1]); /* Was 2.0. */
tthrshld = .01 # tau;

/* -------------------------------------------------------------------
   Define objective function. */

start f(x) global(n, maxm, tau, tthrshld, gender, age, m, dmthd, hdat,
   findcells, lambdatau, lambdaab, gamma, nmcells, tcell, cellage, obsdetec,
   expctdetec, cellprob, malecubs, femalecubs);

/* Initialize things. */

malecubs = 0;
femalecubs = 0;
obsdetec = j(nmcells, 8, 0);
cellprob = j(n # nmcells, 8, 0);
expctdetec = j(nmcells, 8, 0);
cellage = j(n, nmcells, 0);

/* Log-likelihood function.  Return the negative since nlpnms is a
   minimizer.  Model is described in Hwang and Chao (2002). */

/* Pre-compute lambda_ij, and fix the values of phi and gamma. */

lambda = j(n, maxm, 0);
phi = 1.;
gamma = 1.;
do i = 1 to n;
   id = hdat[i,1];
   gval = gender[id];
   parmstrt = gval # 4;
      
   do j = 1 to m[id];

      ageval = age[id,j];

      lambda[i,j] = x[9] + x[10] # dmthd[id,j];
      lambda[i,j] = lambda[i,j] +
         x[ageval] # (1 - gval) + x[parmstrt + ageval] # gval;

      if lambda[i,j] > 500. then lambda[i,j] = 500.;
      lambda[i,j] = exp(lambda[i,j]);
   end;
end;

/* Log-likelihood calculation. */

f = 0;
do i = 1 to n;
   id = hdat[i,1];
   mi = m[id];
   tdiff = hdat[i,(mi + 1)] - hdat[i,2];
   do j = 1 to mi;
      if hdat[i,mi] < tcell[nmcells,2] then
      f = f + log(lambda[i,j]);
   end;

   /* Integrate lambda(t) over the observation interval. */

   intlambda1 = lambda[i,1];
   intlambda = intlambda1;
   do j = 2 to mi;
      deltat = hdat[i,(j + 1)] - hdat[i,j];
      intlambda = intlambda + lambda[i,j] # deltat;
   end;
   deltat = tau - hdat[i,(mi + 1)];
   intlambda = intlambda + lambda[i,mi] # deltat;
 
   li = mi # log(gamma) + (mi - 1) # log(phi) +
      (phi - 1.) # gamma # intlambda1 - phi # gamma # intlambda;

   pi = 1. - exp(-gamma # intlambda);

   if pi < 1.e-7 then pi = 1.e-7;
   li = li - log(pi); 

   if hdat[i,mi] < tcell[nmcells,2] then f = f + li;

   if findcells = 1 then do;
      lambdatau[i] = intlambda;
      
      if tdiff > tthrshld then lambdaab[i] = intlambda;
      else lambdaab[i] = 1.e4;

      gval = gender[id];
      ageval = age[id,1];
      pop = (gval # 4) + ageval;
      if pop = 1 then femalecubs = femalecubs + 1;
      if pop = 5 then malecubs = malecubs + 1;

      /* Compute the number of lions observed in each combination of "tcell"
         and population wherein "age" is the lion's age within a cell. */

      ageval = age[id,mi];
      do j = 1 to nmcells;
         cellage[i,j] = ageval;
      end;
      do j = 1 to mi;
         timeval = hdat[i,(j + 1)];
         thecell = 0;
         do k = 1 to nmcells;
            if tcell[k,1] <= timeval & timeval <= tcell[k,2] then do;
               thecell = k;
               cellage[i,thecell] = age[id,j];
            end;
         end;
         ageval = cellage[i,thecell];
         pop = (gval # 4) + ageval;
         obsdetec[thecell,pop] = obsdetec[thecell,pop] + (1 / mi);
      end;

      /* Compute the probability that this lion will be sighted in each of
         the cells.  Do this by first computing the cumulative intensity
         and then subtracting its value at the previous cell from the value
         at the current cell. */

      clambda = j(nmcells, 1, 0);
      clambda[1] = 0.;
      j = 1;
      do k = 1 to mi;
         if hdat[i,(k + 1)] <= tcell[1,2] then do; 
            if k = 1 then do;
               deltat = hdat[i,2] - tcell[1,1];
            end;
            else if k > 1 then do;
               deltat = hdat[i,(k + 1)] - hdat[i,k];
            end;
            clambda[1] = clambda[1] + lambda[i,k] # deltat;
            if k = mi then do;
               deltat = tau - hdat[i,(k + 1)];
               clambda[1] = clambda[1] + lambda[i,k] # deltat;
            end;
         end;
      end;
      do j = 2 to nmcells;
         clambda[j] = clambda[j - 1];
         do k = 1 to mi;
            if tcell[j,1] <= hdat[i,(k + 1)] &
               hdat[i,(k + 1)] <= tcell[j,2] then do; 
               if k = 1 then do;
                  deltat = hdat[i,2] - tcell[j,1];
               end;
               else if k > 1 then do;
                  deltat = hdat[i,(k + 1)] - hdat[i,k];
               end;
               clambda[j] = clambda[j] + lambda[i,k] # deltat;
               if k = mi then do;
                  deltat = tau - hdat[i,(k + 1)];
                  clambda[j] = clambda[j] + lambda[i,k] # deltat;
               end;
            end;
         end;
      end;
      clambda[nmcells] = max(clambda[nmcells], lambdatau[i]);

      /* Compute cell probabilities for this lion. */

      cellrow = (i - 1) # nmcells + 1;
      ageval = cellage[i,1];
      pop = (gval # 4) + ageval;
      cellprob[cellrow, pop] = 1. - exp(-clambda[1]);
      do j = 2 to nmcells;
         cellrow = (i - 1) # nmcells + j;
         ageval = cellage[i,j];
         pop = (gval # 4) + ageval;
         cellprob[cellrow,pop] = exp(-clambda[j - 1]) - exp(-clambda[j]);
      end;
   end;
end;

if findcells = 1 then do;
   do j = 1 to nmcells;
      do k = 1 to 8;
         do i = 1 to n;
            cellrow = (i - 1) # nmcells + j;
            expctdetec[j,k] = expctdetec[j,k] + cellprob[cellrow,k];
         end;
      end;
   end;
end;

return (-f);
finish f;

/* Declare optimization problem size. */

nmparms = 10;

/* Initial parameter values and bounds constraints. */

x =j(nmparms, 1, 0);
con = j(2, nmparms, 0);
do i = 1 to nmparms;
   x[i] = 0.;
   con[1,i] = -100.;
   con[2,i] = 100.;
end;

/* Declare the number of chi-squared GOF categories. */

nmcells = 6;
lambdatau = j(n, 1, 0);
lambdaab = j(n, 1, 0);

/* Set optimization routine options and run the optimization. */

optn = j(1, 10, .);
optn[2] = 3;
tc = j(1, 10, .);
tc[1] = 1000000;
tc[2] = 1000000;

findcells = 0;
call nlpnms(rc, xres, "f", x`, optn, con, tc);
print "nlpnms return code= " rc;
quit;

/*
c = {"b0" "b_occsn" "b_gender" "b_age" "b_ga" "g_0" "g_occsn" "g_radio"};
print xres[colname=c format=10.3];
*/

/* Compute the model goodness-of-fit, Monte Carlo Chi-squared statistic
   and its p-value.  First, compute the observed test statistic value. */

findcells = 1;
likeld = f(xres);

diffd = j(nmcells, 8, 0);
obsteststat = 0;
do i = 1 to nmcells;
   do j = 1 to 8;
      obsval = obsdetec[i,j];
      expctval = expctdetec[i,j];
      diff = obsval - expctval;
      diffd[i,j] = diff;
      if expctval > .1 then do;
         diff = diff # diff / expctval;
         obsteststat = obsteststat + diff;
      end;
   end;
end;

/*
print "obsteststat= " obsteststat;
print "obsdetec:" obsdetec;
print "expctdetec:" expctdetec;
print "diffd:" diffd;
*/

/* Simulate nmc data sets from the estimated model and compute the test
   statistic for each. */

call randseed(12494472);
nmc = 1000;
simteststat = j(nmc, 1, 0);
do mc = 1 to nmc;
   simdetec = j(nmcells, 8, 0);
   do i = 1 to n;
      id = hdat[i,1];
      mi = m[i];
      gval = gender[id];
      do k = 1 to nmcells;
         cellrow = (i - 1) # nmcells + k;
         ageval = cellage[id,k];
         pop = (gval # 4) + ageval;
         call randgen(dev, "Uniform");
         if dev < cellprob[cellrow, pop] then do;
            simdetec[k,pop] = simdetec[k,pop] + 1;
         end;
      end;
   end;

   /* Compute the test statistic with this simulated data set. */

   do i = 1 to nmcells;
      do j = 1 to 8;
         simdiff = simdetec[i,j] - expctdetec[i,j];

         if expctdetec[i,j] > .1 then do;
            simdiff = simdiff # simdiff / expctdetec[i,j];
            simteststat[mc] = simteststat[mc] + simdiff;
         end;
      end;
   end;
end;

chimean = mean(simteststat);
chistd = std(simteststat);
print "chimean= " chimean " chistd= " chistd;

/* Compute the p-value. */

pval = 0;
do i = 1 to nmc;
   if simteststat[i] < obsteststat then pval = pval + 1;
end;
pval = 1 - pval / nmc;
print "MC Chi-squared GOF statistic= " obsteststat " p-value= " pval;

/* ----------------------- Sex Ratios ------------------------- */

abund = j(5, 2, 0);
sexratio = j(5, 1, 0);

/* Birth sex ratios: males, then females. */

oneyear = '01Jan15'd - '01Jan14'd;
mage = ('01Apr14'd - '01Jan14'd) / oneyear;

lm = 1. - exp(-exp(xres[5] + xres[9]) # oneyear);
lnc = 1. - exp(-exp(xres[6] + xres[9]) # oneyear);
a1 = -log(lm) + mage # log(lnc);
lambda0 = exp(-a1);
nm = ncol(loc(gender = 1));
abund[1,2] = nm / (1. - exp(-lambda0));
abundval = abund[1,2];

lm = 1. - exp(-exp(xres[1] + xres[9]) # oneyear);
lnc = 1. - exp(-exp(xres[2] + xres[9]) # oneyear);
a1 = -log(lm) + mage # log(lnc);
lambda0 = exp(-a1);
nf = n - nm;
abund[1,1] = nf / (1. - exp(-lambda0));
abundval = abund[1,1];

/* Cub through adult sex ratios.  The cub sex ratio is computed using
   only first detection data. */

u = 13;
pi = malecubs / (malecubs + femalecubs);

print "N_m= " malecubs " N_f= " femalecubs " pi= "  pi;

malecubs = malecubs + pi # u;
femalecubs = femalecubs + (1 - pi) # u;

abund[2,2] = malecubs;
abund[2,1] = femalecubs;

/* Juvenile through adult sex ratios are computed on the last encounter
   occasion. */

do i = 1 to n;
   id = hdat[i,1];
   mi = m[id];
   gval = gender[id];
   ageval = age[id,mi];
   lambval = lambdaab[i];
   if ageval > 1 then do;
      abund[(ageval + 1),(gval + 1)] = abund[(ageval + 1),(gval + 1)] +
         (1. / (1. - exp(-gamma # lambdaab[i])));
   end;
end;
totabund = sum(abund[2:5,]);

print "Age Male_Abundance Female_Abundance Sex_Ratio";
do j = 1 to 5;
   abundm = abund[j, 2];
   abundf = abund[j, 1];
   sexratio[j] = abundm / abundf;
   sratio = sexratio[j];
   print j abundm abundf sratio;
end;
print "Total abundance= " totabund;

print "--- Delete-d Jackknife Parameter Confidence Intervals ---";

obsval = j((nmparms + 6), 1, 0);
do i = 1 to nmparms;
   obsval[i] = xres[i];
end;
obsval[nmparms + 1] = sexratio[1];
obsval[nmparms + 2] = sexratio[2];
obsval[nmparms + 3] = sexratio[3];
obsval[nmparms + 4] = sexratio[4];
obsval[nmparms + 5] = sexratio[5];
obsval[nmparms + 6] = totabund;

njck = 250; /* Was 250 */
parmest = j(njck, (nmparms + 6), 0);
x = t(xres);
n = round(.9 # obsn);
ids = 1:obsn;
do ijck = 1 to njck;
   jackids = sample(ids, n, "WOR");
   hdat = obshdat[jackids,];

   optn = j(1, 10, .);
   optn[2] = 0;
   findcells = 0;
   call nlpnms(rc, xres, "f", x`, optn, con, tc);

   do i = 1 to nmparms;
      parmest[ijck,i] = xres[i];
   end;

   /* Compute cell probabilities. */

   findcells = 1;
   likeld = f(xres);

   /* Sex ratios. */

   abund = j(5, 2, 0);
   sexratio = j(5, 1, 0);

   /* Birth sex ratios: males, then females. */

   lm = 1. - exp(-exp(xres[5] + xres[9]) # oneyear);
   lnc = 1. - exp(-exp(xres[6] + xres[9]) # oneyear);
   a1 = -log(lm) + mage # log(lnc);
   lambda0 = exp(-a1);

   nm = 0;
   do i = 1 to n;
      id = hdat[i,1];
      if gender[id] = 1 then nm = nm + 1;
   end;

   abund[1,2] = nm / (1. - exp(-lambda0 # mage));
   abundval = abund[1,2];

   lm = 1. - exp(-exp(xres[1] + xres[9]) # oneyear);
   lnc = 1. - exp(-exp(xres[2] + xres[9]) # oneyear);
   a1 = -log(lm) + mage # log(lnc);
   lambda0 = exp(-a1);
   nf = n - nm;
   abund[1,1] = nf / (1. - exp(-lambda0 # mage));
   abundval = abund[1,1];

   /* Cubs. */

   u = 13;
   pi = malecubs / (malecubs + femalecubs);
   malecubs = malecubs + pi # u;
   femalecubs = femalecubs + (1 - pi) # u;

   abund[2,2] = malecubs;
   abund[2,1] = femalecubs;

   /* Juvenile through adult sex ratios are computed using the age at
      last sighting. */

   do i = 1 to n;
      id = hdat[i,1];
      gval = gender[id];
      mi = m[id];
      ageval = age[id,mi];
      lambval = lambdaab[i];
      if ageval > 1 then do;
         abund[(ageval + 1),(gval + 1)] = abund[(ageval + 1),(gval + 1)] +
            (1. / (1. - exp(-gamma # lambdaab[i])));
      end;
   end;

   do i = 1 to 5;
      parmest[ijck, nmparms + i] = abund[i, 2] / abund[i, 1];
   end;
   totabund = (obsn / n) # sum(abund[2:5,]);
   parmest[ijck, nmparms + 6] = totabund;
end;

cilevel = .95;
nfrac = round(cilevel * njck);
print "No. Parameter estimate Cilow           Cihigh";
do i = 1 to nmparms + 6;

   /* First, sort these parameter estimate values. */

   srtparms = parmest[,i];
   call sort(srtparms);

   /* Find shortest interval. */
   
   minlength = 1.e6;
   cilow = srtparms[1];
   cihigh = srtparms[njck];
   do j = 1 to (njck - nfrac);
      cilength = srtparms[j + nfrac] - srtparms[j];
      if cilength < minlength then do;
         minlength = cilength;
         cilow = srtparms[j];
         cihigh = srtparms[j + nfrac];
      end;
   end;
   parest = obsval[i];
   print i "    " parest "      " cilow "      " cihigh;
end;
quit;
